"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
describe('tform', () => {
    test('basic transforming', () => {
        const record = {
            job: 'Engineer ',
            name: 'John Doe',
            hobbies: 'Biking; Skating;;',
            address: {
                city: 'Cupertino',
                zip: null,
            },
        };
        const rules = {
            job: (X) => X.job(),
            name: {
                first: (X) => X.name().split(' ')[0],
                last: (X) => X.name().split(' ')[1],
            },
            age: (X) => X.age(-1),
            hobbies: (X) => index_1.splitList(X.hobbies()),
            address: {
                city: (X) => X.address().city,
                zip: (X) => X.address().zip,
            },
        };
        const expected = {
            job: 'Engineer',
            name: {
                first: 'John',
                last: 'Doe',
            },
            age: -1,
            hobbies: ['Biking', 'Skating'],
            address: {
                city: 'Cupertino',
                zip: null,
            },
        };
        const tform = new index_1.Tform(rules);
        const output = tform.transform(record);
        expect(output).toEqual(expected);
    });
    test('basic error handling', () => {
        const rules = {
            error: () => {
                throw Error('oh noes!');
            },
            missing: (X) => X.foo(),
        };
        const record1 = { foo: 1 };
        const record2 = {};
        const tform = new index_1.Tform(rules);
        expect(tform.transform(record1)).toEqual({ missing: 1 });
        expect(tform.transform(record2)).toEqual({ missing: undefined });
        expect(tform.getErrors()).toEqual([
            {
                error: Error('oh noes!'),
                field: 'error',
                record_id: undefined,
                record_no: 1,
                record_raw: { foo: 1 },
            },
            {
                error: Error('oh noes!'),
                field: 'error',
                record_id: undefined,
                record_no: 2,
                record_raw: {},
            },
            {
                error: Error("property 'missing' of result is undefined"),
                field: 'missing',
                record_id: undefined,
                record_no: 2,
                record_raw: {},
            },
        ]);
    });
    test('error reporting of record id', () => {
        const rules = {
            missing: (X) => X.foo(),
        };
        const record1 = { pk: 1 };
        const record2 = {};
        const tform = new index_1.Tform(rules, 'pk');
        expect(tform.transform(record1)).toEqual({});
        expect(tform.transform(record2)).toEqual({});
        expect(tform.getErrors()).toEqual([
            {
                error: Error("property 'missing' of result is undefined"),
                field: 'missing',
                record_id: 1,
                record_no: 1,
                record_raw: { pk: 1 },
            },
            {
                error: TypeError("Missing ID key 'pk'"),
                record_no: 2,
                record_raw: {},
            },
            {
                error: Error("property 'missing' of result is undefined"),
                field: 'missing',
                record_id: undefined,
                record_no: 2,
                record_raw: {},
            },
        ]);
    });
});
//# sourceMappingURL=test.js.map