export * from './deserializer';
export * from './utility';
export declare type JSONPrimitive = number | boolean | string | null;
export declare type JSONValue = JSONPrimitive | IJSONArray | IJSONRecord;
export interface IJSONArray extends Array<JSONValue> {
}
export interface IJSONRecord {
    [key: string]: JSONValue;
}
export declare type Rule<Record> = JSONPrimitive | ((record: Record) => any) | IRulesInternal<Record>;
export interface IRulesInternal<Record> {
    [key: string]: Rule<Record>;
}
export declare type Defaultable<Record> = {
    [key in keyof Record]: (fallback?: any) => Record[key];
};
export declare type IRules<Record> = IRulesInternal<Defaultable<Record>>;
export interface ITformError {
    error: Error;
    field?: string;
    record_no: number;
    record_raw: IJSONRecord;
    record_id?: JSONValue;
}
export declare class Tform<Record> {
    private rules;
    private idKey;
    private errors;
    private recordCount;
    constructor(rules: IRules<Record>, idKey?: string | undefined);
    private processRules(rules, results, record);
    transform(record: IJSONRecord): IJSONRecord;
    getErrors(): ITformError[];
    private verifyHasID(record);
    private extractID(record);
    private addError(error, record, field?);
}
