"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
__export(require("./deserializer"));
__export(require("./utility"));
function isPrimitive(obj) {
    return ['string', 'number', 'boolean'].indexOf(obj) >= 0 || obj === null;
}
function wrapRecord(record) {
    return new Proxy(record, {
        get(target, name) {
            const value = target[name];
            return (fallback) => (value !== undefined ? value : fallback);
        },
    });
}
class Tform {
    constructor(rules, idKey) {
        this.rules = rules;
        this.idKey = idKey;
        this.errors = [];
        this.recordCount = 0;
    }
    processRules(rules, results, record) {
        Object.keys(rules).forEach((key) => {
            const rule = rules[key];
            try {
                if (isPrimitive(rule)) {
                    results[key] = rule;
                }
                else if (_.isFunction(rule)) {
                    const wrappedRecord = wrapRecord(record);
                    results[key] = rule(wrappedRecord);
                }
                else {
                    results[key] = {};
                    this.processRules(rule, results[key], record);
                }
                if (results[key] === undefined) {
                    throw Error(`property '${key}' of result is undefined`);
                }
                if (_.isString(results[key])) {
                    results[key] = results[key].trim();
                }
            }
            catch (e) {
                this.addError(e, record, key);
            }
        });
    }
    transform(record) {
        this.recordCount += 1;
        this.verifyHasID(record);
        const results = {};
        this.processRules(this.rules, results, record);
        return results;
    }
    getErrors() {
        return this.errors;
    }
    verifyHasID(record) {
        if (this.idKey && !record[this.idKey]) {
            this.addError(TypeError(`Missing ID key '${this.idKey}'`), record);
        }
    }
    extractID(record) {
        return this.idKey ? record[this.idKey] : undefined;
    }
    addError(error, record, field) {
        this.errors.push({
            error,
            field,
            record_no: this.recordCount,
            record_raw: record,
            record_id: this.extractID(record),
        });
    }
}
exports.Tform = Tform;
//# sourceMappingURL=index.js.map